# Spider usage examples:

from net import Spider


# create a Spider object for specified url
url = "https://www.python.org/"
spider = Spider(url)


# You can user Spider.find_xxx() to find information in the web page

# find all links in the page
link_urls = spider.find_links()
print('links:', link_urls)

# find the links after 'Latest News'
link_urls = spider.find_links('Latest News')
print('links:', link_urls)


# find the links 'Latest News' and the link's url contains "blog."
link_urls = spider.find_links('Latest News', contains="blog.")
print('links:', link_urls)


# find the urls of the images in the page
img_urls = spider.find_images()
print('images:', img_urls)

if len(img_urls) > 0:
    # create a new spider to download the first image url, save image to filename 'pic.xxx'
    # (file extension will be added automatically)
    filename = Spider(url, img_urls[0]).download("pic")
    print('file %s downloaded' % repr(filename))

print("-----------------------------------")

# find the codes in the page
codes = spider.find_codes()
if codes:
    print('code:\n', codes[0])

print("-----------------------------------")

# find the list items after 'Latest News'
words = spider.find_list_items('Latest News')
print('Latest News:', words)

# find the text of the paragraph after 'Download'
text = spider.find_paragraph('Download')
print('Download paragraph: ', text)

tables = Spider("https://www.w3school.com.cn/tags/tag_table.asp").find_tables(text_only=True)
if len(tables) > 0:
    print('table 0:', tables[0].to_list())


# Advanced find example
#
# understanding the structure of webpage's HTML source code, find words in the HTML

# example: find the text of menu items
begin = ['<ul', 'menu']      # find '<ul' tag and 'menu' class as the beginning
end = ['</ul>']              # find '</ul>' tag as the ending
# word is the menu item text
before = ['<li', '<a', '>']  # find '<li' and '<a' and '>' which is before the word
after = ['</a>']             # find '</a>' which is after the word
words = spider.find_list(before, after, begin, end)
print("menus:", words)   # ['Python', 'PSF', 'Docs', 'PyPI', 'Jobs', 'Community']

# find the text of menu items and its links
# word1 is the link
before1 = ['<li', '<a', 'href="']  # find '<li' and '<a' and 'href="' which is before the word1
after1 = ['"']     # find '"' which is after the word1

# word2 is the menu text
before2 = ['>']  # find '>' which is before the word2, after word1
after2 = ['</a>']  # find '</a>' which is after the word2

# compose a list definition, each item of the list is a word define (before, after)
betweens = [(before1, after1), (before2, after2)]

# perform finding
words_list = spider.find_words_list(betweens, begin, end)
print("menus2:", words_list)  # words_list will be a list, each item is a list of two words
# the result is  [['/', 'Python'], ['/psf-landing/', 'PSF'], ...




# find_form
forms = Spider("http://www.baidu.com").find_forms()
# form = HtmlForm(words[0])
print(forms[0].inputs)
# print('form', words)


#
# ZhiDao, BaiKe are the Chinese wiki searching engine of baidu.com.
#
# :Chinese:
# ZhiDao(知道)、BaiKe(百科) 是百度的搜索频道
# ZhiDao类是Spider的子类， 简化了百度知道的爬行、搜索、提取信息
# BaiKe类是Spider的子类， 简化了百度百科的爬行、搜索、提取信息
#

from net import ZhiDao, BaiKe


# 搜索百度百科 "爱在深秋", 对于多义词，辅助关键字是"谭咏麟"
b = BaiKe("爱在深秋", "谭咏麟")
print(b.catalog)  # 取得目录
if '歌词' in b:   # 如果目录中有 '歌词'
    print(b['歌词'])  # 取得 '歌词' 的文字
print(BaiKe("爱在深秋", "谭咏麟")['歌词'])  # 简写为一句话
print("-----------------------------")

# 搜索百度百科 "静夜思"
b = BaiKe("静夜思")
print(b['全文', '原文'])  # 取得目录中的 全文 或 原文 段落文字
print(BaiKe("静夜思")['全文', '原文'])  # 简写为一句话
print("")
print("-----------------------------")

b = ZhiDao("李白 出生地")
print(b.count(), "条答案")
print(b.answer())
print(ZhiDao("李白 出生地").answer())  # 简写为一句话
print("")





#
# WebImage usage example
#

from net import WebImage


imgs = WebImage('black hole', 'png', count=5)
print(imgs.download_all())

# download images one by one
for index, img in enumerate(imgs.images):
    print(img.filename, img.file_ext, img.url)
    if not img.download(index):
        print("error downloading image of %s" % index)




import sys
import re


# replacement of builtins.print() function, compatible in python 2 and python 3.
def rprint(*args,  **kwargs):
    """
    replacement of builtins.print() function, compatible in python 2 and python 3.

    :Chinese: 打印（兼容Python2, Python3)

    :param args:  arguments to be printed

    :param kwargs:  Optional keyword arguments: <br>
        sep:   string inserted between values, default a space.<br>
        end:   string appended after the last value, default a newline.<br>
        flush: whether to forcibly flush the stream.<br><br>
    :return: None
    """
    sep = kwargs.pop('sep', ' ')
    for index, arg in enumerate(args):
        if index > 0:
            sys.stdout.write(sep)
        sys.stdout.write(str(arg))

    end = kwargs.pop('end', '\n')
    if end:
        sys.stdout.write(end)

    flush = kwargs.pop('flush', None)
    if flush:
        sys.stdout.flush()


class StrUtil:
    """
    Utilities for string process.

    :Chinese: 字符串处理类
    """

    @staticmethod
    def sys_encoding():
        """
        return system encoding.

        :Chinese: 取得系统编码

        :return: str
        """
        # noinspection PyBroadException
        try:
            import locale
            lo = locale.getdefaultlocale()
            if lo[1] == 'cp936':  # Simplified Chinese, 简体中文
                return 'gbk'
            elif lo[1] == 'cp950':  # Traditional Chinese, 繁体中文
                return 'cp950'
            elif lo[1] == 'cp936':
                return 'gbk'
        except Exception:
            return 'gbk'

    @staticmethod
    def trim_quote(s):
        """
        Trim quote marks at the beginning and ending of the string.

        :Chinese: 删除字符串两端的引号 (包括单引号、双引号)

        :param s: the string
        :return: str
        """
        if isinstance(s, str):
            if len(s) >= 2:
                if s.startswith('"') and s.endswith('"'):
                    return s[1:len(s)-1]

                if s.startswith("'") and s.endswith("'"):
                    return s[1:len(s)-1]
        return s

    @staticmethod
    def match_offset_length(text: str, findings: [str, list, tuple], start_offset=0):
        """
        Match the finding condition in the string, return matched offset, length.

        :Chinese: 查找字符串, 返回匹配的offset和长度length。

        :param text:     The string
        :param findings:  Find condition, could be a sub string, or a list of str for continuous finding,
                or a tuple of str for finding one of them.<br>
                :Chinese: 查找条件，可以是一个子字符串，或一个字符串list(表示连续查找、and关系),
                或一个字符串tuple(表示查找其中一个、or关系)
        :param start_offset:    (optional) the offset to start finding.<br>
                :Chinese: (可选)起始查找位置, 默认值为0.
        :return: return a tuple of (offset, length).  If not found, offset is -1.
                :Chinese: 返回一个tuple(offset, length)。如果找不到，tuple 中的 offset = -1
        """
        if findings is None:
            return start_offset, 0

        if text is None:
            return -1, 0

        if not isinstance(text, str):
            text = str(text)

        # convert findings to list
        if not isinstance(findings, list) and not isinstance(findings, tuple):
            findings = [str(findings)]

        is_or = True if isinstance(findings, tuple) else False
        offset_begin = -1
        offset_length = 0

        for item in findings:
            if item is None or item == '':
                continue

            if isinstance(item, list) and not isinstance(item, tuple):
                offset, length = StrUtil.match_offset_length(text, item, start_offset)
                offset_begin = offset if offset_begin < 0 <= offset else offset_begin

            elif isinstance(item, str):
                if item.startswith('<re>'):  # regular expression
                    pattern = item[4:]
                    m = re.search(pattern, text[start_offset:])
                    if m is None:
                        offset = -1
                        length = 0
                    else:
                        offset = start_offset + m.start()
                        length = m.end() - m.start()
                        offset_begin = offset if offset_begin < 0 <= offset else offset_begin
                else:
                    offset = text.find(item, start_offset)
                    length = len(item) if offset >= 0 else 0
                    offset_begin = offset if offset_begin < 0 <= offset else offset_begin
            else:
                raise TypeError('finding item type %s invalid' % type(item))

            if is_or:
                if offset >= 0:
                    # return offset, length
                    if offset < offset_begin:
                        offset_begin = offset
                        offset_length = length
            else:
                if offset < 0:
                    return -1, 0
                else:
                    start_offset = offset + length

        if is_or:
            return offset_begin, offset_length
        else:
            # return start, 0
            return offset_begin, start_offset - offset_begin

    @staticmethod
    def match(text, findings, start_offset=0):
        """
        Whether the string match the finding condition.

        :Chinese: 判断字符串是否符合条件, 返回True, False

        :param text:  the string
        :param findings: Find condition, could be a sub string, or a list of str for continuous finding,
                or a tuple of str for finding one of them.<br>
                :Chinese: 查找条件，可以是一个子字符串，或一个字符串list(表示连续查找、and关系),
                或一个字符串tuple(表示查找其中一个、or关系)
        :param start_offset:    (optional) the offset to start finding.<br>
                :Chinese: (可选)起始查找位置, 默认值为0.
        :return: return True if matched, else return False.
        """
        offset, length = StrUtil.match_offset_length(text, findings, start_offset)
        return offset >= 0

    @staticmethod
    def get_word(text, before, after, begin=0, return_offset=False):
        """
        Get a word from text, the word is between before condition, and after condition.

        :Chinese: 从文本中提取一个词，该词在查找条件before后, 查找条件after前

        :param text:    the string
        :param before:   the texts which is before the finding word, could be a string or a list of string.<br>
                :Chinese: 查找在 after 之后的词
        :param after:   the texts which is after the finding word,, could be a sub string or a list of string.<br>
                :Chinese: 查找条件end
        :param begin:    offset to start finding.
                :Chinese:（可选）开始查找的位置
        :param return_offset:  whether return offset, boolean.
                :Chinese:（可选）是否返回查找后的位置
        :return: return str if found, return None if not found.
                if return_offset is True, return a tuple of (str, offset_int)
                :Chinese: 返回一个词。 如果找不到，返回None
        """
        if begin is None:
            begin = 0

        if begin is not None and not isinstance(begin, int):
            offset, length = StrUtil.match_offset_length(text, str(begin), 0)
            if offset < 0:
                return None, None if return_offset else None
            else:
                begin = offset + length

        if begin < 0:
            if return_offset:
                return None, begin
            else:
                return None

        b, len1 = StrUtil.match_offset_length(text, before, begin)
        if b > 0:
            e, len2 = StrUtil.match_offset_length(text, after, b + len1)
            if e > b + len1:
                if return_offset:
                    return text[b + len1: e], e + len2
                else:
                    return text[b + len1: e]
        else:
            if return_offset:
                return None, begin
            else:
                return None

    @staticmethod
    def get_word_list(text, begin, betweens, end=None, start_offset=0):
        """
        get a list of keywords from the text.

        :Chinese: 从文本中, 查找出一串单词列表。

        :param text:   the text to process
        :param begin:  begin condition, could be a sub string or a list of string.<br>
        :param betweens:  words definition list
                each item is a keyword definition tuple, which contains a before and after condition.
                Example:
                <code>
                    [
                      (['<li', '<a', '>'], '</a'),
                      (['author', '>'], '</a')
                    ]
                </code>
                define two keywords.

                :Chinese: betweens 是一个单词定义列表。
                         每一个元素单词定义, 一个 2个元素tuple, 第0个元素是before条件，
                         第1个元素是after条件.

        :param end:  (optional) end condition, could be a sub string or a list of string.<br>
        :param start_offset: (optional) offset to start finding
        :return:  return a list of words
        """
        result = []
        if begin:
            offset, length = StrUtil.match_offset_length(text, begin, start_offset)
        else:
            offset = 0

        if end:
            end_offset, _ = StrUtil.match_offset_length(text, end, offset)
        else:
            end_offset = -1

        while offset >= 0:
            words = []
            # search words
            for item in betweens:
                before = item[0]
                after = item[1]
                word, offset = StrUtil.get_word(text, before, after, offset, return_offset=True)

                if offset > end_offset > 0:
                    word = None

                if word:
                    words.append(word)
                else:
                    offset = -1
                    break
            # add words to result
            if words and len(words) == len(betweens):
                result.append(words)

        if len(betweens) == 1:
            ret = []
            for item in result:
                if len(item) == 1:
                    ret.append(item[0])
            return ret
        else:
            return result

    @staticmethod
    def is_json_string(s):
        """
        Whether the string is a JSON string.

        :Chinese: 判断 字符串s 是否是一个json字符串

        :param s: the string
        :return: True or False
        """
        if isinstance(s, str):
            s = s.strip()
            if s.startswith('{') and s.endswith('}'):
                return True
        return False

    @staticmethod
    def is_xml_string(s):
        """
        Whether the string is a XML string.

        :Chinese: 判断 字符串s 是否是一个 xml 字符串

        :param s: the string
        :return: True or False
        """
        if isinstance(s, str):
            s = s.strip()
            if s.startswith('<') and s.endswith('>'):
                return True
        return False

    @staticmethod
    def find_word(text, finding, index=0, sep=None):
        """
        Find a word ing a line.

        :Chinese: 在一行文字中查找符合条件的词.

        :param text:   the string
        :param index:  the index of occurrence , the first is 0, second is 1 ...
                    :Chinese: 第几个. 0表示第一个， 1表示第二个
        :param finding: (optional) finding condition, could be a sub string or a list of string.<br>
                :Chinese:(可选)查找条件，可以是一个子字符串，或一个字符串列表
        :param sep:   separate char, default is blank char.<br>
                :Chinese:(可选)词分割符，默认为空格
        :return: return str if found, return None if not found.
                :Chinese: 如果找到，返回字符串。如果找不到，返回None
        """
        count = 0
        if isinstance(text, str):
            words = StrUtil.split(text, sep)
            for word in words:
                if StrUtil.match(word, finding):
                    if count == index:
                        return word
                    count += 1
        return None

    @staticmethod
    def find_sep(text, sep, start_offset=0):
        """
        find separator char

        :Chinese: 查找分割符

        :param text:  the string
        :param sep:   the separator char
        :param start_offset: 
        :return: return a tuple of (offset, length_of_sep)
        """
        if isinstance(sep, str):
            return text.find(sep, start_offset), len(sep)
        elif isinstance(sep, list) or isinstance(sep, tuple):
            for word in sep:
                ret = text.find(word, start_offset)
                if ret >= 0:
                    return ret, len(word)
        return -1, 0

    @staticmethod
    def is_blank_char(c):
        """ Whether the c is blank char """
        return c == ' ' or c == '\t' or c == '\r'

    @staticmethod
    def split(text, sep=None):
        """
        split the text into word list by specified separator

        :param text:  the text
        :param sep:  separator string
        :return: return a list of str
        """
        if sep is None:
            sep = [' ', '\t']

        if text is None:
            text = ''
        if not isinstance(text, str):
            text = str(text)

        words = []
        length = len(text)
        offset = 0
        word = ''
        while offset < length:
            c = text[offset:offset+1]
            offset += 1
            if c in sep:
                if word == '' and StrUtil.is_blank_char(c):
                    pass
                else:
                    words.append(word)
                    word = ''
            else:
                word += c
        if word:
            words.append(word)
        return words

    @staticmethod
    def split2(text, sep, first=True):
        """
        Split the text into two-word list.

        :Chinese: 将一个字符串切分为两个词

        :param text:  the text
        :param sep:   separator string
        :param first: (optional) If True, when separator is missing, set the first word with text and
                leave the second word empty. If False, vise versa.
        :return: return a 2-word list
        """
        if text is None:
            text = ''

        pos, length = StrUtil.find_sep(text, sep)

        if pos >= 0:
            s1 = text[:pos]
            s2 = text[pos + length:]
            return s1.strip(), s2.strip()
        else:
            if first:
                return text.strip(), ''
            else:
                return '', text.strip()

    @staticmethod
    def split_number_unit(s):
        """
        Split the string, such as '2ms', into number and unit.

        :Chinese: 将一个形如: 2ms 的字符串切分为 数字,单位

        :param s: the string
        :return: return a tuple of (number, unit)
        """
        if s is None:
            s = ''
        if not isinstance(s, str):
            s = str(s)

        number = ''
        unit = ''
        for c in s:
            if unit == '':
                if ('0' <= c <= '9') or c == '.':
                    number += c
                elif len(number) == 0 and c == '-':
                    number = c
                else:
                    unit += c
            else:
                unit += c
        if len(number) > 0:
            number = float(number) if '.' in number else int(number)
        return number, unit.strip()

    @staticmethod
    def grep(text, finding):
        """
        Split the text into lines, return the lines matched the finding condition.

        :Chinese: 将文本切分为多行, 返回符合查找条件的行

        :param text:  the string
        :param finding: the finding condition, could be a sub string, or a list of string.
                :Chinese: 查找条件，可以是一个子字符串，或一个字符串列表
        :return: return a list of str， each element is a line matched the finding condition.
                :Chinese: 返回一个字符串列表， 每个元素是一个符合查找条件的行
        """
        result = []

        # check each line in the text
        if text is None:
            return result

        if isinstance(text, list) or isinstance(text, tuple):
            lines = text
        else:
            text = str(text) if not isinstance(text, str) else text
            lines = text.split('\n')

        if isinstance(lines, list) or isinstance(lines, tuple):
            for line in lines:
                if StrUtil.match(line, finding):
                    result.append(line)

        return result

    @staticmethod
    def is_ip_number(text):
        """
        Whethe the string is a valid IP number, which range is [0-255]

        :Chinese: 判断文字是否是有效的IP数字范围 [0-255]

        :param text: the string
        :return:  True of False
        """
        # noinspection PyBroadException
        try:
            n = int(text)
            if isinstance(text, float):
                return False
            return 0 <= n <= 255
        except Exception:
            return False

    @staticmethod
    def get_words(text, line_findings, word_findings, index=0, multiple=False, number_unit=False):
        lines = StrUtil.match(text, line_findings)
        ret = []
        for line in lines:
            word = StrUtil.find_word(line, word_findings, index)
            if word:
                if number_unit:
                    word, unit = StrUtil.split_number_unit(word)
                if not multiple:
                    return word
                else:
                    ret.append(word)
        return ret




# Net ping and scan examples

from net import Net

# ping a server(or an IP)
t = Net.ping("www.bing.com")  # return milliseconds, return -1 means not available
print('milliseconds', t)

# get IP address of current computer
my_ip = Net.local_ip()

# create an IP range (a list of IP address)
ip_list = Net.ip_range(my_ip, 1, 100)
print(ip_list)

# Scan the IPs, return a list of pingable IPs
exists_ips = Net.ip_scan(ip_list)
print(exists_ips)

# Scan the IPs, return a list of not-pingable IPs
exists_ips = Net.ip_scan(ip_list, exists=False)
print(exists_ips)

# Scan the IPs, return a list, whose item is a tuple of (ip, mac_address)
exists_ips = Net.ip_scan(ip_list, mac=2)
print(exists_ips)

# Scan the IPs, return a list, whose item is a tuple of (ip, mac_address, manufacturer)
exists_ips = Net.ip_scan(ip_list, mac=3)
print(exists_ips)


# detect whether specified port of specified IP is opened
if Net.is_port_open(my_ip, 80):
    print('port 80 of', my_ip, 'opened')
else:
    print('port 80 of', my_ip, 'not opened')


# scan a list of port on specified IP address, return opened port list
port_list = Net.port_scan(my_ip, [80, 8080, 21, 22, 443, 445])
print('opened ports', port_list)



#
# Spider, get web page, extract word from the content, download file.
#
# ::
#     # Spider usage examples:
#
#     from net import Spider
#
#
#     # create a Spider object for specified url
#     url = "https://www.python.org/"
#     spider = Spider(url)
#
#
#     # You can user Spider.find_xxx() to find information in the web page
#
#     # find all links in the page
#     link_urls = spider.find_links()
#     print('links:', link_urls)
#
#     # find the links after 'Latest News'
#     link_urls = spider.find_links('Latest News')
#     print('links:', link_urls)
#
#
#     # find the links 'Latest News' and the link's url contains "blog."
#     link_urls = spider.find_links('Latest News', contains="blog.")
#     print('links:', link_urls)
#
#
#     # find the urls of the images in the page
#     img_urls = spider.find_images()
#     print('images:', img_urls)
#
#     if len(img_urls) > 0:
#         # create a new spider to download the first image url, save image to filename 'pic.xxx'
#         # (file extension will be added automatically)
#         filename = Spider(url, img_urls[0]).download("pic")
#         print('file %s downloaded' % repr(filename))
#
#     print("-----------------------------------")
#
#     # find the codes in the page
#     codes = spider.find_codes()
#     if codes:
#         print('code:\n', codes[0])
#
#     print("-----------------------------------")
#
#     # find the list items after 'Latest News'
#     words = spider.find_list_items('Latest News')
#     print('Latest News:', words)
#
#     # find the text of the paragraph after 'Download'
#     text = spider.find_paragraph('Download')
#     print('Download paragraph: ', text)
#
#     tables = Spider("https://www.w3school.com.cn/tags/tag_table.asp").find_tables(text_only=True)
#     if len(tables) > 0:
#         print('table 0:', tables[0].to_list())
#
#
#     # Advanced find example
#     #
#     # understanding the structure of webpage's HTML source code, find words in the HTML
#
#     # example: find the text of menu items
#     begin = ['<ul', 'menu']      # find '<ul' tag and 'menu' class as the beginning
#     end = ['</ul>']              # find '</ul>' tag as the ending
#     # word is the menu item text
#     before = ['<li', '<a', '>']  # find '<li' and '<a' and '>' which is before the word
#     after = ['</a>']             # find '</a>' which is after the word
#     words = spider.find_list(before, after, begin, end)
#     print("menus:", words)   # ['Python', 'PSF', 'Docs', 'PyPI', 'Jobs', 'Community']
#
#     # find the text of menu items and its links
#     # word1 is the link
#     before1 = ['<li', '<a', 'href="']  # find '<li' and '<a' and 'href="' which is before the word1
#     after1 = ['"']     # find '"' which is after the word1
#
#     # word2 is the menu text
#     before2 = ['>']  # find '>' which is before the word2, after word1
#     after2 = ['</a>']  # find '</a>' which is after the word2
#
#     # compose a list definition, each item of the list is a word define (before, after)
#     betweens = [(before1, after1), (before2, after2)]
#
#     # perform finding
#     words_list = spider.find_words_list(betweens, begin, end)
#     print("menus2:", words_list)  # words_list will be a list, each item is a list of two words
#     # the result is  [['/', 'Python'], ['/psf-landing/', 'PSF'], ...
#
#
#
#


from html.parser import HTMLParser
import re
import json
import os

from .util import StrUtil
from .http import Http, Url


# A class to parse HTML to text
class DeHTMLParser(HTMLParser):
    """
    A class to parse HTML to text

    """
    def __init__(self, **kwargs):
        HTMLParser.__init__(self)
        self._text = []
        self._last_tag = None
        self.tag = kwargs.get('tag', None)
        self.tag_attrs = {}
        self.line_break = kwargs.get('line_break', False)

    def handle_data(self, data):
        if self._last_tag == "script":
            return

        text = data.strip()
        if len(text) > 0:
            if self.line_break:
                text = re.sub('[ \t]+', ' ', text)
            else:
                text = re.sub('[ \t\r\n]+', ' ', text)
            self._text.append(text + ' ')

    def handle_starttag(self, tag, attrs):
        # print(tag, attrs)
        self._last_tag = tag
        if self.tag and tag == self.tag:
            for attr in attrs:
                self.tag_attrs[attr[0]] = attr[1]
        if tag == 'p':
            self._text.append('\n\n')
        elif tag == 'br':
            self._text.append('\n')
        elif tag == 'script':
            pass

    def handle_startendtag(self, tag, attrs):
        self._last_tag = None
        # if tag == 'br':
        #     self.__text.append('\n\n')

    def text(self):
        if self.line_break:
            s = '\n'
        else:
            s = ''
        return s.join(self._text).strip()


# A class to parse HTML form
class HtmlForm(HTMLParser):
    """
    A class to parse HTML to text

    """

    class Input(dict):
        def __init__(self, attrs):
            for attr in attrs:
                self[attr[0]] = attr[1]

    def __init__(self, html_text, **kwargs):
        HTMLParser.__init__(self)
        self.attrs = {}
        self.inputs = []
        if html_text:
            self.feed(html_text)

    def handle_data(self, data):
        pass
        # if self._last_tag == "script":
        #     return
        #
        # text = data.strip()
        # if len(text) > 0:
        #     if self.line_break:
        #         text = re.sub('[ \t]+', ' ', text)
        #     else:
        #         text = re.sub('[ \t\r\n]+', ' ', text)
        #     self._text.append(text + ' ')

    def handle_starttag(self, tag, attrs):
        # print(tag, attrs)
        if tag == 'input':
            self.inputs.append(HtmlForm.Input(attrs))

    def handle_startendtag(self, tag, attrs):
        pass

    def text(self):
        if self.line_break:
            s = '\n'
        else:
            s = ''
        return s.join(self._text).strip()


# convert html to text
def html2text(html, convert=True, line_break=False):
    """
    Convert html to text.

    :Chinese: 将HTML转化为纯文本

    :param html:   HTML string
    :param convert: (optinal) whether convert to pure text
    :param line_break: (optinal) whether keep line breaks
    :return: str
    """
    if not convert:
        return html
    if html is None:
        return None
    # noinspection PyBroadException
    try:
        parser = DeHTMLParser(line_break=line_break)
        parser.feed(html)
        parser.close()
        return parser.text()
    except Exception:
        return html


class Link(list):
    def __init__(self, url, text):
        self.append(url)
        self.append(text)

    @property
    def url(self):
        return self[0] if len(self) > 0 else None

    @property
    def text(self):
        return self[1] if len(self) >= 1 else None


class Table(list):
    class Row(list):
        pass

    class Cell:
        def __init__(self, text, urls=None):
            self.text = text
            self.urls = urls or []
            self.attrs = {}

        def __repr__(self):
            return '<Cell %s>' % repr(self.text)

    def _regulate(self):
        # TODO: process span row, span col
        pass

    def to_list(self):
        self._regulate()
        result = []
        for row in self:
            r = []
            for cell in row:
                r.append(cell.text)
            result.append(r)
        return result

    def __repr__(self):
        result = '[\n'
        count = 0
        for row in self:
            if count > 0:
                result += ',\n'
            if len(row) > 0:
                result += '    ' + str(row)
                count += 1
        result += '\n]'
        return result


# net spider
class Spider:
    """
    Net Spider, get web page, extract word from the content.

    :Chinese: 网络爬虫类, 抓取网页，分析提取其中的数据

    Usage：

    # create a Spider object for url

    w = Spider("https://www.python.org/")

    # find links

    links = w.find(Spider.LINKS)

    """

    BING = "https://cn.bing.com/search?q={0}"
    BAIDU = "https://www.baidu.com/s?wd={0}"

    def __init__(self, url: str = None, sub_url: str = None):
        self.pure_text = True  # whether return pure text

        self._url = Url.join(url, sub_url)
        self._page = None

        if url is None:
            self._url = Spider.BING

    def search(self, word):
        """ search a word """
        self._page = Http.get(self._url, word).text
        return self

    def download(self, filename):
        """ download and save to file """
        return Http.get(self._url, '').save_to_file(filename)

    # noinspection PyMethodMayBeStatic
    def _stop_chars(self):
        """ return stop chars """
        chars = [
            '，',
            '；',
            '。',
            '</div',
            '</td',
            '</li',
            '</h',
            '</p',
            ',',
        ]
        return tuple(chars)

    def _ensure_page(self):
        """ ensure self._page has content """
        if self._url and not self._page:
            self._page = Http.get(self._url).text

    def find(self, before, after=None, begin=None, end=None):
        """
        find information on the web page

        :param before: texts which is before the finding text.  could be a string or a list of string.
        :param after: (optional) texts which is after the finding text, could be a string or a list of string
        :param begin: (optional) texts from where to begin the finding, could be a string or a list of string or an offset int
        :param end: (optional) texts from where to end the finding, could be a string or a list of string or an offset int
        :return:  return string
        """
        self._ensure_page()

        if not self._page:
            return None

        if after is None:
            after = self._stop_chars()

        start = 0
        if begin:
            start, _ = StrUtil.match_offset_length(self._page, begin)
        if start < 0:
            return None

        s = StrUtil.get_word(self._page, before, after, start)
        return html2text(s, self.pure_text)

    def find_links(self, begin=None, end=None, contains=None, regex=False):
        """
        find links in the webpage. A link is a tag like '<a href="url">text</a>' in HTML.

        :param begin: (optional) texts from where to begin the finding, could be a string or a list of string or an offset int
        :param end: (optional) texts from where to end the finding, could be a string or a list of string or an offset int
        :param contains: (optional) the substring that the link's url should contains
        :param regex: (optional) whether contains is a regex pattern
        :return: return a list, each item of the list is a tuple of (url, text) of a link.
        """
        self._ensure_page()
        before1 = ['<a', 'href="']
        after1 = '"'
        before2 = '>'
        after2 = '</a'
        betweens = [(before1, after1), (before2, after2)]
        words = self.find_words_list(betweens, begin, end)

        # convert to list of Link
        links = []
        for item in words:
            links.append(Link(item[0], item[1]))

        # filter the links
        if contains:
            result = []
            for u in links:
                if regex:
                    if re.search(contains, u[0]) is not None:
                        result.append(u)
                elif u[0].find(contains) >= 0:
                    result.append(u)
            return result
        else:
            return links

    def find_images(self, begin=None, end=None, contains=None, regex=False):
        """
        find images in the webpage. An image is a tag like '<img src="url">' in HTML.

        :param begin: (optional) texts from where to begin the finding, could be a string or a list of string or an offset int
        :param end: (optional) texts from where to end the finding, could be a string or a list of string or an offset int
        :param contains: (optional) the substring that the image's url should contains
        :param regex: (optional) whether contains is a regex pattern
        :return: return a list, each item of the list is url of image.
        """
        before = ['<img', 'src="']
        after = '"'
        betweens = [[before, after]]
        urls = self.find_words_list(betweens, begin, end)

        # filter the links
        if contains:
            result = []
            for url in urls:
                if regex:
                    if re.search(contains, url) is not None:
                        result.append(url)
                elif url.find(contains) >= 0:
                    result.append(url)
            return result
        else:
            return urls

    def find_codes(self, begin=None, end=None):
        """
        find codes in the webpage. code is the text between '<code>' and '</code>' in HTML

        :param begin: (optional) texts from where to begin the finding, could be a string or a list of string or an offset int
        :param end: (optional) texts from where to end the finding, could be a string or a list of string or an offset int
        :return: return a list, each item of the list is a piece of code.
        """
        before = ['<code', '>']
        after = ['</code', '>']
        betweens = [[before, after]]
        return self.find_words_list(betweens, begin, end, line_break=True)

    def find_paragraph(self, begin=None, end=None):
        """
        find paragraph text in the webpage. A paragraph is a tag like '<p>text</p>' in HTML.

        :param begin: (optional) texts from where to begin the finding, could be a string or a list of string or an offset int
        :param end: (optional) texts from where to end the finding, could be a string or a list of string or an offset int
        :return: return a list, each item of the list is a paragraph text.
        """
        before = '<p>'
        after = '</p'

        # convert begin to start
        start = 0
        if begin:
            start, _ = StrUtil.match_offset_length(self._page, begin)
        if start < 0:
            return None
        s = StrUtil.get_word(self._page, before, after, start)

        return html2text(s, self.pure_text)

    def find_forms(self, begin=None, end=None):
        self._ensure_page()
        text = self._page

        forms = self._get_texts(text, '<form', '</form', begin, end)

        result = []
        for form in forms:
            result.append(HtmlForm(form))
        return result

    def set_text(self, text):
        """ set text of the web page """
        self._page = text
        return self

    def _get_texts(self, page, before, after, begin=None, end=None, append_char='>'):
        """ get text from page """
        result = []
        s, offset = StrUtil.get_word(page, before, after, begin, return_offset=True)
        while s is not None:
            if append_char is not None:
                result.append(before + s + after + append_char)
            else:
                result.append(s)
            s, offset = StrUtil.get_word(page, before, after, offset, return_offset=True)
        return result

    def find_tables(self, begin=None, end=None, text_only=False):
        """
        find paragraph text in the webpage. A paragraph is a tag like '<p>text</p>' in HTML.

        :param begin: (optional) texts from where to begin the finding, could be a string or a list of string or an offset int
        :param end: (optional) texts from where to end the finding, could be a string or a list of string or an offset int
        :return: return a list, each item of the list is table.
            A table is a list, each item is a row. A row is a list each item is a cell.
            A cell is a tuple, first element is cell text, second element is the links in the cell.
            A link is a
        """
        self._ensure_page()
        result = []
        # find tables in '<table> ... </table>'
        tables = self._get_texts(self._page, '<table', '</table', begin, end)
        for table in tables:
            table_obj = Table()
            # find row in '<tr> ... </tr>'
            rows = self._get_texts(table, '<tr', '</tr')
            for row in rows:
                row_obj = Table.Row()
                # find cell in '<td> .. </td>'
                if row.find('<th') >= 0:
                    cells = self._get_texts(row, '<th', '</th')
                else:
                    cells = self._get_texts(row, '<td', '</td')
                # for each cell
                for cell in cells:
                    # get urls in the <td>
                    urls = []
                    if cell.find('<a') >= 0:
                        urls = Spider().set_text(cell).find_links()
                    # get cell text
                    parser = DeHTMLParser(tag='td', line_break=True)
                    parser.feed(cell)
                    cell_text = parser.text()
                    if text_only:
                        cell_obj = Table.Cell(cell_text)
                    else:
                        cell_obj = Table.Cell(cell_text, urls)
                    cell_obj.attrs = parser.tag_attrs
                    # append cell to the row
                    row_obj.append(cell_obj)
                # append row to the table
                table_obj.append(row_obj)
            # append table to tables
            result.append(table_obj)
        return result

    @staticmethod
    def _get_begin_condition(b, prefix):
        # convert word to list
        if prefix and (not isinstance(prefix, list) and not isinstance(prefix, tuple)):
            prefix = [str(prefix)]
        if not prefix:
            prefix = []

        # join word, begin
        if isinstance(b, list) or isinstance(b, tuple):
            b = list(prefix) + list(b)
        elif isinstance(b, str):
            b = list(prefix).append(b)
        elif b is None:
            b = list(prefix)
        else:
            b = list(prefix).append(str(b))

        return b

    def find_list_items(self, begin=None, is_ol=False):
        """
        find list items in the webpage. list items are within '<ul> <li></li> </ul>' in HTML

        :param begin: (optional) texts from where to begin the finding, could be a string or a list of string or an offset int
        :param end: (optional) texts from where to end the finding, could be a string or a list of string or an offset int
        :param is_ol: (optional) whether the list is order list which tag is '<ol>'
        :return: return a list, each item is an item which is a list of [url, text]
        """
        self._ensure_page()
        tag_begin = ['<ul'] if not is_ol else ['<ol']
        end = ['</ul'] if not is_ol else ['</ol']
        begin = self._get_begin_condition(tag_begin, begin)
        # word1 is the link url
        before1 = ['<li', '<a', 'href="']
        after1 = '"'
        # word1 is the text
        before2 = '>'
        after2 = '</a'
        betweens = [(before1, after1), (before2, after2)]
        words_list = self.find_words_list(betweens, begin, end)
        if not words_list:
            betweens = [(['<li', '>'], '</li>')]
            words_list = self.find_words_list(betweens, begin, end)
        return words_list

    def find_text(self, before, after=None, begin=None):
        """
        find words

        :param before: texts which is before the finding text.  could be a string or a list of string.
        :param after: (optional) texts which is after the finding text, could be a string or a list of string
        :param begin: (optional) texts from where to begin the finding, could be a string or a list of string or an offset int
        :return:  return string
        """
        self._ensure_page()

        if not self._page:
            return None

        if after is None:
            after = self._stop_chars()

        text = html2text(self._page)
        # print(text)

        start = 0
        if begin:
            start, _ = StrUtil.match_offset_length(text, begin)
        if start < 0:
            return None

        s = StrUtil.get_word(text, before, after, start)
        return html2text(s, self.pure_text)

    def find_list(self, before, after=None, begin=None, end=None):
        """
        find a list of word

        :param before: texts which is before the finding text.  could be a string or a list of string.
        :param after: (optional) texts which is after the finding text, could be a string or a list of string
        :param begin: (optional) texts from where to begin the finding, could be a string or a list of string or an offset int
        :param end: (optional) texts from where to end the finding, could be a string or a list of string or an offset int
        :return:  return string
        """
        self._ensure_page()

        if after is None:
            after = self._stop_chars()

        if self._page:
            betweens = [(before, after)]
            words = StrUtil.get_word_list(self._page, begin, betweens, end=end)
            ret = []
            for item in words:
                ret.append(html2text(item, self.pure_text))
            return ret

    def find_words_list(self, betweens, begin=None, end=None, line_break=False):
        """
        Find a list of words, each item of list may have serveral words.

        :Chinese: 查找一个列表, 每一个列表项有多个词.

        :param betweens:  betweens list, each item is a tuple of (before, after)
        :param begin: (optional) texts from where to begin the finding, could be a string or a list of string or an offset int
        :param end: (optional) texts from where to end the finding, could be a string or a list of string or an offset int
        :param line_break:  (optional) whether keep line breaks
        :return: return a list
        """
        self._ensure_page()

        if not isinstance(betweens, list) and not isinstance(betweens, tuple):
            raise ValueError("betweens must be a list")
        else:
            for item in betweens:
                if not (isinstance(item, list) or isinstance(item, tuple)) or len(item) != 2:
                    msg = "item of betweens must be a list of two elements: before, after"
                    raise ValueError(msg)

        if self._page:
            words = StrUtil.get_word_list(self._page, begin, betweens, end=end)
            if self.pure_text:
                for index, item in enumerate(words):
                    if isinstance(item, tuple):
                        item = list(item)
                        words[index] = item
                    if isinstance(item, list):
                        for i in range(0, len(item)):
                            item[i] = html2text(item[i], line_break=line_break)
                    elif isinstance(item, str):
                        words[index] = html2text(item, line_break=line_break)

            return words

    @property
    def text(self):
        self._ensure_page()
        return html2text(self._page)


# BaiDu Chinese Wiki 百度百科
class BaiKe(Spider):
    """
    百度百科
    ===========
    使用方法示例：

    # 创建对象, 百科条目是: 爱在深秋，
    # (可选)辅助注释是：谭咏麟
    b = BaiKe("爱在深秋", "谭咏麟")

    # 打印内容目录
    print( b.catalog )

    # 如果目录中有歌词, 打印歌词内容
    if '歌词' in b:
        print(b['歌词'])

    """

    def __init__(self, word=None, finding=None):
        self.base_url = "https://baike.baidu.com"
        super().__init__(self.base_url + "/item/{0}")
        self._explain = None  # 解释
        self._catalog = []  # 百科的目录
        self._synonyms = []  # 同义词数据

        if word:
            self.load(word, finding)

    @property
    def explain(self):
        """ 解释文字 """
        return self._explain

    @property
    def catalog(self):
        """ 目录列表 """
        return self._catalog

    def _get_explain(self):
        if self._page:
            self._explain = StrUtil.get_word(self._page, ['<meta', '"description"', 'content="'], ['">'])
            self._load_catalog()

    def _load_synonyms(self):
        if self._page:
            betweens = [
                (['<li', '<a', "href='"], "'>"),
                (None, '</a')
            ]
            others_list = StrUtil.get_word_list(self._page, 'polysemantList-wrapper', betweens)
            if len(others_list) == 0:
                betweens = [(['<li', '<a', 'href="'], '"'), ('>', '</a')]
                others_list = StrUtil.get_word_list(self._page, 'custom_dot', betweens)
            for item in others_list:
                if isinstance(item, list) and len(item) == 2:
                    self._synonyms.append([item[1], self.base_url + item[0]])

    def _load_url(self, url):
        self._page = Http.get(url)
        self._get_explain()

    def _load_catalog(self):
        if self._page:
            betweens = [
                (['<li', '<a', 'href=', '>'], '</a'),
            ]
            # begin = '<div class="catalog-list'
            begin = 'catalog-list'
            self._catalog = StrUtil.get_word_list(self._page, begin,
                                                  betweens, end='anchor-list')

    def _find_catalog(self, item):
        """ find a item in the catalog """
        if isinstance(item, tuple):
            for e in item:
                r = self._find_catalog(e)
                if r >= 0:
                    return r
            return -1
        else:
            for index, elem in enumerate(self._catalog):
                if elem == item:
                    return index
            for index, elem in enumerate(self._catalog):
                if StrUtil.match(elem, item):
                    return index
        return -1

    def load(self, word, finding=None):
        """
        读取一个词的百科

        :param word:  一个词
        :param finding: （可选)限定语，用于寻找同义词
        :return: 返回对象本身
        """
        self.search(word)
        self._get_explain()
        self._load_synonyms()

        if finding:
            self.find_others(finding)
        return self

    def find_others(self, finding):
        """ 寻找同义词,  如找到则返回对象本身, 如找不到返回None"""
        for item in self._synonyms:
            if StrUtil.match(item[0], finding):
                self._load_url(item[1])
                return self
        if len(self._synonyms) > 0 and self._page.find('custom_dot') > 0:
            item = self._synonyms[0]
            self._load_url(item[1])
            return self
        return None

    def count(self):
        """ 返回同义词的数量 """
        return len(self._synonyms)

    def others(self, index):
        """
        跳转到指定序号index的同义词。

        :param index:  序号index
        :return: 返回一个BaiKe对象，指向序号index的同义词
        """
        if 0 <= index < len(self._synonyms):
            item = self._synonyms[index]
            obj = BaiKe()
            obj._load_url(item[1])
            obj._synonyms = self._synonyms
            obj.word = item[0]
            return obj
        else:
            raise IndexError('index %s out of bound' % index)

    def __contains__(self, item):
        return self._find_catalog(item) >= 0

    def __getitem__(self, item):
        if item is None or item == '':
            index = 0
        else:
            index = self._find_catalog(item)

        if index < 0:
            raise StopIteration()

        paragraph = self._catalog[index]
        before = ['<div class="para-title', paragraph, '</div>']
        after = '<div class="anchor-list'
        start = 0
        text = StrUtil.get_word(self._page, before, after, start)

        # 如果找不到
        if len(html2text(text)) < 5:
            # 接下一个段落
            before.append(after)
            before.append('</div>')
            text = StrUtil.get_word(self._page, before, after, start)
        # 如果找到
        if text:
            text = text.replace("</div>", "</div><br>")
        return html2text(text, self.pure_text)


# BaiDu Chinese Wiki 百度知道
class ZhiDao(Spider):
    """
    百度知道
    =======
    使用方法示例：

    # 创建对象, 提出问题: 李白 出生地
    b = ZhiDao("李白 出生地")

    # 答案的数量
    b.count()

    # 打印第0条答案
    print( b.answer(0) )

    """

    def __init__(self, word):
        super().__init__("https://zhidao.baidu.com/search?word={0}")
        self._answers = []  # 答案, 每一个元素是一个tuple, 第一个元素是标题，第二个是url
        self.search(word)

    def answer(self, index=0):
        """
        返回答案文字

        :param index (可选)第几条答案
        """
        if 0 <= index < len(self._answers):
            item = self._answers[index]
            # title = item[0]
            page = Http.get(item[1])  # url = item[1]
            before = ['<div', 'answer', '>', 'content-container', '>']
            after = ['<div', 'quality-content']
            word = StrUtil.get_word(page, before, after)
            return html2text(word)

    def count(self):
        """ 返回答案的数量 """
        return len(self._answers)

    def search(self, word):
        """ 提出一个问题 """
        self._answers = []
        super().search(word)
        if self._page:
            begin = 0
            betweens = [
                (['<dt', 'result', '<a', 'href="'], '"'),
                (['>'], '</a'),
            ]
            words = StrUtil.get_word_list(self._page, begin, betweens)
            for item in words:
                if len(item) == 2:
                    self._answers.append((html2text(item[1]), item[0]))
        return self


# ImageData
class ImageData:
    """ ImageData"""

    def __init__(self, data):
        self.url = None  # 图片rul
        self.thumb_url = None  # 缩略图rul
        self.filename = None  # 文件扩展名
        self.file_ext = None  # 文件名
        self.ref_url = None  # 图片源头的网页url
        self.title = None  # 图片源头的网页标题
        self._load(data)

    # noinspection PyMethodMayBeStatic
    def valid_format(self, fmt):
        """ fmt 是否是有效的图片格式 """
        if fmt in ['.jpg', '.jpeg', '.gif', '.bmp', '.png', '.webp']:
            return True
        return False

    def valid(self):
        """ 图片是否有效 """
        if self.file_ext:
            return True
        return False

    def _set_format(self, fmt):
        if fmt:
            fmt = fmt.lower()
            fmt = ImageData._cut_char(fmt, '?')
            fmt = ImageData._cut_char(fmt, '!')
            # if fmt.find('?') >= 0:
            #     fmt = fmt[:fmt.find('?')]
            # if fmt.find('!') >= 0:
            #     fmt = fmt[:fmt.find('!')]
            # if fmt == '.jpeg':
            #     fmt = '.jpg'

            if self.valid_format(fmt):
                self.file_ext = fmt
            else:
                self.file_ext = None

    def _load(self, data):
        """ 从 data 中读入图片数据 """
        if isinstance(data, dict):
            if 'murl' in data:
                self.url = data['murl']
                self._set_format(ImageData.get_file_ext(self.url))
                self.filename = ImageData.get_file_name(self.url)
            if 'turl' in data:
                self.thumb_url = data['turl']
            if 't' in data:
                self.title = data['t']
            if 'purl' in data:
                self.ref_url = data['purl']

    @staticmethod
    def _cut_char(url, c):
        """ 切除 url 中 字符c 后面的部分 """
        if url.find(c) >= 0:
            url = url[:url.find(c)]
        return url

    @staticmethod
    def get_file_ext(url):
        """ 取得 url 中的文件扩展名 """
        url = ImageData._cut_char(url, '?')
        url = ImageData._cut_char(url, '!')

        pos = url.rfind('.')
        if pos >= 0:
            url = url[pos:]
            return url
        else:
            return ''

    @staticmethod
    def get_file_name(url):
        """ 取得 url 中的文件名 """
        url = ImageData._cut_char(url, '?')
        url = ImageData._cut_char(url, '!')

        pos = url.rfind('/')
        if pos >= 0:
            url = url[pos + 1:]

        return url

    def download(self, filename=None):
        """
        下载图片, 保存到文件.

        :param filename: (可选)存盘文件名.
                文件名可以不带扩展名， 如: file1,
                本函数将根据图片类型自动添加扩展名， 并返回实际存盘的文件名， 如: file1.jpg。

        :return: 如果失败，则返回None。<br>
                如果成功存盘，返回存盘文件名。<br>
                如果参数filename缺省，则不存盘，返回图片数据(bytes)。
        """
        # noinspection PyBroadException
        try:
            response = Http.get(self.url)
        except Exception:
            return None

        if not response:
            return

        if filename:
            # noinspection PyBroadException
            try:
                filename = str(filename)
                ext = ImageData.get_file_ext(filename)
                if ext != self.file_ext:
                    filename += self.file_ext
                f = open(filename, 'wb')
                f.write(response.content)
                f.close()
                return filename
            except Exception:
                pass
        else:
            return response.content


mod_requests = None


# import requests package
def get_requests():
    """ import requests package """
    global mod_requests

    if mod_requests is None:
        mod_requests = __import__('requests')

    return mod_requests


# bing.com image search
class WebImage(Spider):
    """
    bing.com image search

    :Chinese: 图片搜索 (使用 bing.com)
    ==============
    Usage:

    # Create object，search for png image of baby, image count is 5
    b = WebImage('baby', 'png', count=5)

    # download all images to subdirectory images
    b.download_all("images")

    # another download style: for each image
    for index, img in enumerate(b.images):

        # print url, filename, file extension of the image
        print(img.url, img.filename, img.file_ext)

        # download image to file
        img.download('images/' + str(index))

    """
    MAX_IMAGES = 50  # max count of images for one search

    def __init__(self, word, file_ext=None, size=None, count=20, first=0):
        url = 'https://cn.bing.com/images/async?q={0}&first={1}&count={2}&relp={3}&lostate=r&mmasync=1'
        super().__init__(url)
        self.word = ''
        self.images = []
        if word:
            self.load(word, file_ext, size, count, first)

    # noinspection PyMethodMayBeStatic
    def _parse_json(self, json_str):
        # noinspection PyBroadException
        try:
            data = json.loads(json_str)
            return data
            pass
        except Exception:
            pass

        data = {}
        # noinspection PyBroadException
        try:
            s = json_str.strip()
            s = s.replace(',', '\n')
            s = s.replace('{', '')
            s = s.replace('}', '')
            lines = s.split('\n')
            for line in lines:
                if line.find(':') > 0:
                    key, value = StrUtil.split2(line, ':')
                    key = StrUtil.trim_quote(key)
                    value = StrUtil.trim_quote(value)
                    data[key] = value
        except Exception:
            pass
        return data

    def load(self, word, file_ext=None, size=None, count=20, first=0):
        """
        search keyword

        :param word:    keyword
        :param file_ext: (optional) file extension of image, such as 'jpg' 'png'  'gif'
        :param count:    (optional) count of image
        :param first:   (optional) skip first some images
        :return: self
        """
        # process file extension
        if file_ext:
            file_ext = str(file_ext).lower()
            if not file_ext.startswith('.'):
                file_ext = '.' + file_ext
            search_word = word + ' ' + file_ext
        else:
            search_word = word

        self.word = word
        self.images = []

        # create a session using requests
        req = get_requests()
        session = req.Session()

        retry = 1
        while len(self.images) < count and retry < 10000:
            retry += 1
            # get web page
            number_per_page = WebImage.MAX_IMAGES
            url = Http._compose_get_url(self._url, [search_word, first, number_per_page, number_per_page])
            r = session.get(url=url, timeout=(3.05, 10))
            self._page = r.text

            first += number_per_page

            # analysis web page
            betweens = [
                (['<a class="iusc"', 'm="'], '"')
            ]
            words = StrUtil.get_word_list(self._page, 0, betweens)

            # convert to images
            for word in words:
                word = word.replace('&quot;', '"')
                data = self._parse_json(word)
                img = ImageData(data)
                if img.valid():
                    if file_ext is None or img.file_ext == file_ext:
                        self.images.append(img)
                if len(self.images) >= count:
                    break
        return self

    def count(self):
        """ return count of images """
        return len(self.images)

    def download_all(self, path=None):
        """
        download all image, save file to specified path

        :param path:  save path
        :return: return count of downloaded files
        """
        success = 0
        for index, img in enumerate(self.images):
            if path:
                filename = os.path.join(str(path), str(index))
            else:
                filename = str(index)
            if img.download(filename):
                success += 1
        return success



# Spider usage examples:
import re

from net import Spider

# create a Spider object for specified url
url = "https://www.python.org/"
url = "https://www.qq.com"
spider = Spider(url)
print(spider.text)
# print(str(bytes(spider.text, "utf-8"), encoding="gbk"))
exit()

# get the web-page content
spider = spider.get()

# s = spider.find_tables()
# print(s)
#
# exit()

# You can user find_xxx() to find information in the web page


# tables = Spider("https://www.w3school.com.cn/tags/tag_table.asp").find_tables(text_only=True)
# print(tables)


# find_form
words = Spider("http://www.baidu.com").find_forms()
print('form', words)
exit()

# find the links in the page
link_urls = spider.find_links('Latest News')
print('links:', link_urls)


# find the links whose url contains "blog."
link_urls = spider.find_links('Latest News', contains="blog.")
print('links:', link_urls)


# find the urls of the images in the page
img_urls = spider.find_images()
print('images:', img_urls)

if len(img_urls) > 0:
    # create a new spider to download the first image url, save image to filename 'pic.xxx'
    # (file extension will be added automatically)
    filename = Spider(url, img_urls[0]).download("pic")

    # open image file using PIL
    # from PIL import Image
    # Image.open(filename).show()

print("-----------------------------------")

# find the codes in the page
codes = spider.find_codes()
if codes:
    print('code:\n', codes[0])

print("-----------------------------------")

# find the list items after 'Latest News'
words = spider.find_list_items('Latest News')
print('Latest News:', words)

# find the text of the paragraph after 'Download'
text = spider.find_paragraph('Download')
print('Download paragraph: ', text)

# Advanced find example
#
# understanding the structure of webpage's HTML source code, find words in the HTML

# example: find the text of menu items
begin = ['<ul', 'menu']      # find '<ul' tag and 'menu' class as the beginning
end = ['</ul>']              # find '</ul>' tag as the ending
# word is the menu item text
before = ['<li', '<a', '>']  # find '<li' and '<a' and '>' which is before the word
after = ['</a>']             # find '</a>' which is after the word
words = spider.find_list(before, after, begin, end)
print("menus:", words)   # ['Python', 'PSF', 'Docs', 'PyPI', 'Jobs', 'Community']

# find the text of menu items and its links
# word1 is the link
before1 = ['<li', '<a', 'href="']  # find '<li' and '<a' and 'href="' which is before the word1
after1 = ['"']     # find '"' which is after the word1

# word2 is the menu text
before2 = ['>']  # find '>' which is before the word2, after word1
after2 = ['</a>']  # find '</a>' which is after the word2

# compose a list definition, each item of the list is a word define (before, after)
betweens = [(before1, after1), (before2, after2)]

# perform finding
words_list = spider.find_words_list(betweens, begin, end)
print("menus2:", words_list)  # words_list will be a list, each item is a list of two words
# the result is  [['/', 'Python'], ['/psf-landing/', 'PSF'], ...



#!/usr/bin/python
# coding:utf-8
#
# e-mail sender
#
# ::
#     # Send e-mail example
#
#     from net import Mail
#
#     # Set parameters
#     username = 'xxxxxxx@host.com'  # username to login SMTP server
#     password = 'xxxxxxxxxx'      # password to login SMTP server
#     receiver = 'xxxxxxxx@host.com'  # receiver e-mail address
#
#     mail = Mail(username, password)  # Create mail object
#
#     # Send mail with attachment file '1.jpg'
#     mail.send([receiver], 'My Subject', "This is body", ['1.jpg'])
#
#

import smtplib
from email.mime.text import MIMEText
from email.mime.multipart import MIMEMultipart
from email import encoders
from email.mime.base import MIMEBase
from email.header import Header
import os


class Mail:

    __version__ = '1.0.0'
    """
    A class to send e-mail.

    :Chinese: 发送邮件的类

    ------------------

    # Usage Example:


    # create object

    mail = Mail('sender@host.com', passwd, host='smtp.host.com', sender='sender@host.com')

    # send

    mail.send(['receiver@some.com'], 'the title', 'content text', [filename1, filename2])

    """

    def __init__(self, user, passwd, sender=None, host=None):
        """
        create instance of Mail

        :param user:   username to login SMTP server
        :param passwd: password to login SMTP server
        :param host:   (optional) SMTP server. if empty, detect host by the user parameter.
        :param sender: (optional) sender of the mail. if empty, use the user parameter
        """

        self.user = user      # username to login host
        self.passwd = passwd  # password to login host
        self.host = host      # SMTP server
        self.sender = sender  # sender of the mail

        if sender is None or sender == '':
            self.sender = self.user

        if (host is None or host == '') and user.find('@') > 0:
            host = 'smtp.' + user[user.find('@') + 1:]
            self.host = host

    def send(self, receivers: list, subject: str, body: str, attach_filenames: list = None):
        """
        send a mail

        :param receivers:  list of the e-mail addresses of receivers
        :param subject:    title of the mail
        :param body:    content of the mail
        :param attach_filenames:      (optional) attachment file name list
        
        :return: return True if success, return SMTPException object if failed.
        """
        # SEE: https://www.code-learner.com/python-send-html-image-and-attachment-email-example/

        # validate params
        if isinstance(receivers, str):
            receivers = [receivers]
        if isinstance(receivers, list):
            receivers = '; '.join(receivers)
        else:
            raise ValueError('argument receiver invalid')

        if attach_filenames is None:
            attach_filenames = []

        if isinstance(attach_filenames, str):
            attach_filenames = [attach_filenames]

        if not isinstance(attach_filenames, list):
            raise smtplib.SMTPException('Mail.send() parameter attach_filenames must be a list of filenames')

        # compose mail message
        msg = MIMEMultipart()
        msg['From'] = self.sender
        msg['To'] = receivers
        msg['Subject'] = str(subject)
        msg.attach(MIMEText(str(body), 'html'))

        # add attachment files
        if isinstance(attach_filenames, list):
            count = 0
            for file in attach_filenames:
                file = file.strip(' ')
                if os.path.exists(file):
                    # set attachment mime type
                    mime = MIMEBase("application", "octet-stream")
                    # another example: mime = MIMEBase('image', 'png', filename='img1.png')

                    # read attachment file content into the MIMEBase object
                    mime.set_payload(open(file, "rb").read())

                    # add required header data
                    mime.add_header("Content-Disposition", "attachment",
                                    filename=Header(os.path.basename(file), "utf-8").encode())
                    mime.add_header('X-Attachment-Id', str(count))
                    mime.add_header('Content-ID', '<' + str(count) + '>')  # cid

                    # encode with base64
                    encoders.encode_base64(mime)

                    # add MIMEBase object to MIMEMultipart object
                    msg.attach(mime)

                    count += 1
                else:
                    raise smtplib.SMTPException('Mail.send() attachment file ' + str(file) + ' do not exists')

        smtp_obj = smtplib.SMTP(self.host)
        smtp_obj.login(self.user, self.passwd)
        smtp_obj.sendmail(self.sender, receivers, msg.as_string())
        return True




#
# This net package provides tools to perform net ping, scan port, send email, http, web spider,
# access web API.
#


__version__ = '0.3.2'
__author__ = "JoStudio"
__date__ = "2022/9/1"

from .http import Http
from .scan import Net
from .spider import Spider, BaiKe, ZhiDao, WebImage, ImageData
from .util import StrUtil
from .webapi import WebAPI
from .mail import Mail







import requests

from .http import UserAgent
from .util import StrUtil


# 取得headers中指定name的值
def headers_get(headers, name, sub_name=None):
    """ 取得headers中的名为name的字段的值, sub_name是子字段 """
    name = name.lower()
    for key in headers:
        if key.lower() == name:
            val = headers[key]
            vals = val.split(';')
            if sub_name is None:
                if len(vals) > 0:
                    return vals[0].strip()
            else:
                for item in vals:
                    if item.find('=') > 0:
                        s1, s2 = StrUtil.split2(item, '=')
                        if s1.lower() == sub_name.lower():
                            return s2


# WebAPI
class WebAPI:
    class API:
        def __init__(self, method, url, name, **kwargs):
            self.method = str(method).upper()  # HTTP method, such as 'GET', 'POST', 'DELETE', 'PUT'
            self.url = url        # the URL
            self.name = name      # function name
            self.kwargs = kwargs  # key-value arguments of the function
            self.headers = {}     # headers for request

        def call(self, **kwargs):
            """ call the api """
            req_headers = {"User-Agent": UserAgent.default}
            if isinstance(self.headers, dict):
                for key in self.headers:
                    req_headers[key] = self.headers[key]

            if self.method in ['GET', 'DELETE', 'HEAD']:
                params = kwargs
                post_data = {}
            else:
                params = {}
                post_data = kwargs

            response = requests.request(self.method, self.url, data=post_data, params=params, headers=req_headers)

            if response.status_code == 200:
                content_type = headers_get(response.headers, 'content-type')
                if isinstance(content_type, str):
                    if content_type.find('json') >= 0:
                        return response.json()
                return response.text
            else:
                raise ValueError('response code %s' % response.status_code)

    def __init__(self):
        self._apis = []

    def add_func(self, method, url, func_name, **kwargs):
        """
        Add a API function

        :param method:  HTTP method, such as 'GET', 'POST', 'DELETE', 'PUT'
        :param url:     url
        :param func_name:  function name
        :param kwargs:     key-value arguments of the function
        :return:  None
        """
        api = WebAPI.API(method, url, func_name, **kwargs)
        self._apis.append(api)

    def find_func(self, func_name):
        """
        Find a function
        :param func_name: function name
        :return: return WebAPI.API object if success. raise Exception if not found
        """
        for api in self._apis:
            if api.name == func_name:
                return api

    def call(self, func_name, **kwargs):
        """
        Call a function

        :param func_name:  function name
        :param kwargs:  key-value arguments of the function
        :return:  return the result of the function. raise exception if error occurs
        """
        api = self.find_func(func_name)
        if api:
            return api.call(**kwargs)
        else:
            raise NotImplemented('function %s not implemented' % func_name)

    def help(self, func_name=None):
        """ get help of API function"""
        result = {}
        for api in self._apis:
            if func_name is None or api.name == func_name:
                result[api.name] = api.kwargs
        return result




#!/usr/bin/python
# coding:utf-8
#
# Network ping and scan functions
# ::
#     # Net ping and scan examples
#
#     from net import Net
#
#     # ping a server(or an IP)
#     t = Net.ping("www.bing.com")  # return milliseconds, return -1 means not available
#     print('milliseconds', t)
#
#     # get IP address of this computer
#     my_ip = Net.local_ip()
#
#     # create an IP range (a list of IP address
#     ip_list = Net.ip_range(my_ip, 1, 100)
#     print(ip_list)
#
#     # Scan the IPs, return list of pingable IPs
#     exists_ips = Net.ip_scan(ip_list)
#     print(exists_ips)
#
#     # whether a port of specified IP is opened
#     if Net.is_port_open(my_ip, 80):
#         print('port 80 of', my_ip, 'opened')
#     else:
#         print('port 80 of', my_ip, 'not opened')
#
#
#     # scan a list of port on specified IP address, return opened port list
#     port_list = Net.port_scan(my_ip, [80, 8080, 21, 22, 443, 445])
#     print('opened ports', port_list)
#


import socket
import platform
import subprocess
import threading
import signal
import time

from .util import rprint, StrUtil
from .http import Http


# Internal use：A task for scan
class _ScanTask:
    """
    A task for scan
    :Chinese: 用于批量扫描的任务类
    """
    BLOCK = ''

    cancel = False

    # noinspection PyMethodMayBeStatic
    def block_char(self):
        if not _ScanTask.BLOCK:
            _ScanTask.BLOCK = "▋" if platform.system().lower() == 'windows' else '#'
        return _ScanTask.BLOCK

    def __init__(self, data_list, tag, exists):
        self.data_list = data_list
        self.tag = tag
        self.exists = exists
        self.result = []  # 结果列表
        self.locker = threading.RLock()  # 线程锁
        self.finished = 0   # 完成扫描的IP个数
        self.total = len(self.data_list)  # 需要扫描的IP总数
        self.progress_delta = 5
        _ScanTask.cancel = False
        signal.signal(signal.SIGINT, _ScanTask.handler)
        signal.signal(signal.SIGTERM, _ScanTask.handler)

    @staticmethod
    def handler(signal_num, handler):
        _ScanTask.cancel = True
        rprint("User canceled", end='')
        if signal_num or handler:
            pass

    def append_result(self, s):
        # 添加到结果列表
        self.locker.acquire()
        try:
            self.result.append(s)
        finally:
            self.locker.release()

    def finish_one(self):
        # 完成一个IP的扫描
        self.locker.acquire()
        try:
            self.finished += 1
            self.print_progress()
        finally:
            self.locker.release()

    def print_progress(self):
        # 显示进度
        if (self.finished % self.progress_delta == 0) or self.finished >= self.total:
            percent = self.finished * 1.0 / self.total * 100
            blocks = self.block_char() * int(round(percent / 5))
            msg = "\rScanning %s/%s :  %s %s" % (self.finished, self.total, blocks, int(percent))
            rprint(msg + '%', end='', flush=True)

    # noinspection PyMethodMayBeStatic
    def clear_progress(self):
        # 清除进度显示
        space = " " * 70
        msg = "\r" + space
        rprint(msg, end='', flush=True)
        rprint("\r", end='', flush=True)

    # noinspection PyMethodMayBeStatic
    def get_threads(self, total):
        """ 判断线程数量 """
        max_threads = 20
        threads = int(round(total / 5))
        if threads <= 0:
            threads = 1
        if threads > max_threads:
            threads = max_threads
        return threads


# Internal use：Thread function: scan a range of ip
def _ip_scan_thread(task, start, end):
    for i in range(start, end):
        time.sleep(0.05)
        if _ScanTask.cancel:
            return

        if i >= len(task.data_list):
            continue

        ip = task.data_list[i]  # get an ip
        timeout = 50 if Net.is_lan_ip(ip) else 500  # long timeout for LAN

        # ping twice
        can_ping = Net.ping(ip, 1, timeout) >= 0
        if not can_ping:
            can_ping = Net.ping(ip, 1, timeout * 2) >= 0  # try again

        if can_ping and task.exists:
            # if ping failed, and need result, put it in list
            if task.tag:
                mac_address = Net.get_mac(ip)
                if task.tag == 'list' or task.tag >= 2:
                    if task.tag == 3:
                        t = (ip, mac_address, Net.mac_factory(mac_address))
                    else:
                        t = (ip, mac_address)
                    task.append_result(t)
                else:
                    if mac_address is not None:
                        task.append_result(mac_address)
            else:
                task.append_result(ip)
        elif not can_ping and not task.exists:
            # if ping failed, and need result, put it in list
            task.append_result(ip)

        task.finish_one()  # Notify parent: one ip is finished


# Internal use：Thread function: scan a range of port
def _port_scan_thread(task, start, end):
    """ 内部使用的：线程函数，用于扫描一批IP端口 """
    for i in range(start, end):
        if _ScanTask.cancel:
            return

        if i >= len(task.data_list):
            continue

        ip = task.tag  # 取得IP地址
        port = task.data_list[i]  # 取得一个端口
        port = int(port)
        # timeout = 50 if is_lan_ip(ip) else 500  # 如果是本地IP，则超时设短一些

        # 测试端口是否能连通
        opened = Net.is_port_open(ip, port)
        # rprint('ip', ip, 'port', port, ' open=', opened)

        if opened and task.exists:
            # 如果端口能连通，且需要连通结果，则加入结果列表中
            task.append_result(port)
        elif not opened and not task.exists:
            # 如果端口不能连通，且需要连不通的结果，则加入结果列表中
            task.append_result(port)

        task.finish_one()  # 完成一个


# Network ping and scan functions
class Net:
    """
    Network ping and scan

    """

    @staticmethod
    def is_ip(text):
        """
        Whether the string is an IPV4 string.

        :Chinese: 判断指定文字是否是一个 IP 地址

        :param text: the string
        :return: True or False
        """
        if text is None:
            text = ''
        arr = text.split('.')
        if len(arr) != 4:
            return False
        for item in arr:
            if not StrUtil.is_ip_number(item):
                return False
        return True

    @staticmethod
    def ip_set(ip, index, number):
        """
        Change ip address number at specified index to specified number.

        :Chinese: 更改 IP地址 指定位置的数值

        :param ip: IPv4 address string
        :param index:  the index of the number, first is 0, valid indexes are 0, 1, 2, 3
                :Chinese: 第几位， 0表示第一位，取值范围是 0, 1, 2, 3
        :param number:   the number , the value range is [0-255]
                :Chinese: 数值(整数)，取值范围是 0 - 255
        :return: return IP address string. return None if error occurs.
                :Chinese: 返回更改后的IP地址。如果出错，返回None
        """
        if ip is None:
            raise TypeError('ip is None')

        arr = ip.split('.')
        if 0 <= index < len(arr):
            if StrUtil.is_ip_number(number) or number == '*':
                arr[index] = str(number)
                return '.'.join(arr)
            else:
                raise ValueError('number must in the range of [0 - 255]')
        else:
            raise ValueError('index must be 0 or 1 or 2 or 3')

    @staticmethod
    def ip_get(ip, index):
        """
        Get ip address number at specified index.

        :Chinese: 获取 IP地址中 指定位置的数值

        :param ip:   IPv4 address string
        :param index:  the index of the number, first is 0, valid indexes are 0, 1, 2, 3
                :Chinese: 第几位， 0表示第一位，取值范围是 0, 1, 2, 3
        :return: return ip address number at specified index. return None if error occurs.
                :Chinese: 返回IP地址中第 index 位的数值(整数)。如果出错，返回None
        """
        if ip is None:
            raise TypeError('ip is None')

        arr = ip.split('.')
        if 0 <= index < len(arr):
            return int(arr[index])
        else:
            raise ValueError('index invalid')

    @staticmethod
    def ip_range(ip, start=None, end=None, index=3):
        """
        Define an IP address range, return  a list of IP address string.

        :Chinese: 定义一个IP地址范围, 返回一个IP地址列表.

        :param ip:    IPv4 address string
        :param start:  start number
        :param end:    end number
        :param index:  (optional)the index of the number, first is 0, valid indexes are 0, 1, 2, 3
                :Chinese: 第几位， 0表示第一位，取值范围是 0, 1, 2, 3
        :return: return  a list of IP address string.
                :Chinese: 返回一个IP地址列表
        """
        if start is None and end is None:
            arr = ip.split('.')
            if len(arr) != 4:
                raise TypeError('%s is not a IP address' % ip)
            if arr[3] == '*':
                ip2 = Net.ip_set(ip, 3, 1)
                return Net.ip_range(ip2, 1, 255)
            elif arr[3].find('-') > 0:
                s1, s2 = StrUtil.split2(arr[3], '-')
                ip2 = Net.ip_set(ip, 3, 1)
                return Net.ip_range(ip2, int(s1), int(s2))
            elif Net.is_ip(ip):
                return [ip]
            else:
                raise ValueError('%s invalid ip' % ip)
        else:
            if Net.is_ip(ip):
                if not (0 <= start <= 255):
                    raise ValueError('start number %s is not in range 0-255')
                if not (0 <= end <= 255):
                    raise ValueError('end number %s is not in range 0-255')
                if not (2 <= index <= 3):
                    raise ValueError('index %s is not in range 2-3')
                arr = ip.split('.')
                result = []
                for i in range(start, end+1):
                    if index == 3:
                        arr[index] = str(i)
                        result.append('.'.join(arr))
                    elif index == 2:
                        arr[index] = str(i)
                        for n in range(1, 255):
                            arr[index + 1] = str(n)
                            result.append('.'.join(arr))
                return result
            raise TypeError('%s is not a IP address' % ip)

    @staticmethod
    def port_range(port_start, port_end=None):
        """
        Define a port range, return  a list of port int.

        :Chinese: 定义一个端口范围, 返回一个端口列表.

        :param port_start:  starting port number, int, value range is [1-65535]
                    :Chinese: 起始端口(整数), 取值范围是 1-65535
        :param port_end:  ending port number, int, value range is [1-65535]
                    :Chinese:结束端口(整数), 取值范围是 1-65535
        :return: return  a list of port int.
        """
        if isinstance(port_start, str) and port_end is None:
            if port_start == 'commons' or port_start == "common":
                return Port.common_ports()

            if port_start.find('-') >= 0:
                s1, s2 = StrUtil.split(port_start, '-')
                port_start = int(s1)
                port_end = int(s2)
            elif port_start.find(',') >= 0:
                arr = port_start.split(',')
                ret = []
                for item in arr:
                    item = item.strip()
                    ret.append(int(item))
                return ret
            else:
                port_start = int(port_start)
                port_end = port_start

        if not isinstance(port_start, int) or not (0 <= port_start < 65536):
            raise ValueError('port %s is out of range [1, 65535]' % port_start)
        if not isinstance(port_end, int) or not (0 <= port_end < 65536):
            raise ValueError('port %s is out of range [1, 65535]' % port_end)
        if port_end < port_start:
            port_start, port_end = port_end, port_start

        result = []
        for i in range(port_start, port_end + 1):
            result.append(i)
        return result

    @staticmethod
    def get_connection_ip(host, port):
        """
        Get the local IP address that connects to the specified host.

        :Chinese: 取得可连接到指定服务器的本地IP地址.

        :param host:  host domain or ip
        :param port:  the port
        :return: return the local IP address that connects to the specified host.
                :Chinese: 返回连接指定服务器的本地IP地址.
        """
        sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        # noinspection PyBroadException
        try:
            sock.connect((host, port))
            host = sock.getsockname()
            sock.close()
            return host[0]
        except Exception:
            if sock:
                sock.close()
            return None

    @staticmethod
    def local_ip():
        """
        Get local IP address.

        :Chinese: 取得局域网IP地址.

        :return: return IP address string
        """
        my_ip = Net.get_connection_ip("qq.com", 80)  # try internet
        if my_ip is None:
            my_ip = Net.get_connection_ip("10.0.0.1", 80)
            if my_ip is None:
                my_ip = Net.get_connection_ip("192.168.1.1", 80)
        return my_ip

    @staticmethod
    def has_internet():
        """ Whether internet is available """
        my_ip = Net.get_connection_ip("qq.com", 80)  # try internet
        return my_ip is not None

    @staticmethod
    def is_lan_ip(ip):
        """ Whether specified IP address is LAN IP address """
        arr = ip.split('.')
        if arr[0] == '192':
            if len(arr) >= 1 and arr[1] == '168':
                return True
        elif arr[0] == '172':
            if len(arr) >= 1 and ('16' <= arr[1] <= '31'):
                return True
        elif arr[0] == '10':
            return True
        elif ip == '127.0.0.1':
            return True
        return False

    @staticmethod
    def run_command(cmd_line):
        """
        run a command in current OS

        :param cmd_line:  command line
        :return: return a tuple (ret, text), the first element is return value，the second is result text.
        """

        # 创建一个进程，运行命令
        p = subprocess.Popen(cmd_line,
                             stdout=subprocess.PIPE,
                             stderr=subprocess.PIPE,
                             shell=True
                             )
        p.wait()

        # 取得返回值
        ret_code = p.returncode
        out = ''

        # noinspection PyBroadException
        try:
            # 取得返回文字
            out = p.stdout.read()
            p.stdout.close()
            p.stderr.close()
        except Exception:
            pass

        # 转换文字编码
        encode = StrUtil.sys_encoding() if platform.system().lower() == 'windows' else 'utf-8'
        # encode = "gbk" if platform.system().lower() == 'windows' else 'utf-8'

        # noinspection PyBroadException
        try:
            text = out.decode(encode)
            if not isinstance(text, str):
                text = str(text)
        except Exception:
            # noinspection PyBroadException
            try:
                text = out.decode('ascii')
                if not isinstance(text, str):
                    text = str(text)
            except Exception:
                text = out

        return ret_code, text

    @staticmethod
    def ping(ip, times=1, timeout=1000):
        """
        Pint specified IP address

        :param ip:  IP address string or host name
        :param times:  (optional) times of ping
        :param timeout:  (optional) waiting timeout in milliseconds，defalut is 1000.
        :return: return time usage of ping in millisecond.  return -1 when ping fails.
            :Chinese: 返回Ping通时间（单位为毫秒)。 返回-1表示Ping不通。
        """

        # compose a command line : "ping -c 1 bing.com"
        if platform.system().lower() == 'windows':
            command = 'ping -w %s -n %s %s' % (timeout, times, ip)
        else:
            timeout = round(timeout / 10) / 100
            command = 'ping -W %s -c %s %s' % (timeout, times, ip)

        code, text = Net.run_command(command)

        # find lines which has 'time' word
        lines = StrUtil.grep(text, ('时间', 'time'))

        # get time from response text
        time_usages = []
        for line in lines:
            word = StrUtil.find_word(line, ('时间', 'time'))  # find word with time
            _, s_time = StrUtil.split2(word, ('=', '<'))  # find word  after '=' or '<'
            number, unit = StrUtil.split_number_unit(s_time)  # split into number, unit
            if isinstance(number, int) or isinstance(number, float):
                if times == 1:
                    return number
                elif number >= 0:
                    time_usages.append(number)

        if len(time_usages) == 0:
            return -1
        else:
            # for multi time value, calculate the average time
            total = 0
            for number in time_usages:
                total += number
            return round(total / len(time_usages), 2)

    @staticmethod
    def get_mac(ip):
        """
        Get the MAC address of specified IP address.

        :Chinese: 取得 IP地址 对应的 MAC地址。

        :param ip:  IP address string.
        :return: return MAC address string, like: 78:44:fd:6d:1c:91". return empty string if fail.
                :Chinese: 返回MAC地址字符串。如："78:44:fd:6d:1c:91"
        """
        # command line: "arp -a 192.168.1.1"
        if platform.system().lower() == 'windows':
            command = 'arp -a %s' % ip
            delimiter = '-'
        else:
            # cmd = 'arp -a %s' % ip
            command = 'arping -c 1 %s' % ip
            delimiter = ':'

        code, text = Net.run_command(command)

        # get MAC from response text
        lines = StrUtil.grep(text, ip)  # find lines with ip
        for line in lines:
            # rprint(delimiter, type(line), line)
            word = StrUtil.find_word(line, [delimiter, delimiter])
            if word:
                if word.find('--') >= 0:
                    continue
                if delimiter == '-':
                    word = word.replace(delimiter, ':')
                if word.startswith('[') and word.endswith(']'):
                    word = word[1: len(word) - 1]
                return word

        return ''

    @staticmethod
    def mac_factory(mac):
        """
        Get manufacture factory name by specified MAC address.

        :Chinese: 根据mac码，获取制造工厂名称.

        :param mac:  the MAC address string
        :return: return a string of manufacture factory name. return empty string if fail.
            :Chinese: 返回字符串（制造工厂名称)
        """
        url = 'https://mac.bmcx.com/{0}__mac/'
        mac = mac.replace(':', '-')
        page = Http.get(url, mac)
        name = StrUtil.get_word(page, ['组织名称', '<td', '>'], ['</td'])
        if name is not None:
            return name
        else:
            url = 'http://www.coffer.com/mac_find/?string={0}'
            page = Http.get(url, mac)
            name = StrUtil.get_word(page, ['Vendor</th', '<tr', '<td', '<td', '<a', '>'], ['</a'])
            return name if name is not None else ''

    @staticmethod
    def ip_scan(ip_list, mac=False, exists=True, threads=0):
        """
        Scan IP address list, return a list of ip address or MAC address which is pingable.

        :Chinese: 扫描 IP地址列表，取得 可PING通的 IP地址列表 (或MAC地址列表).

        :param ip_list:   the IP address list
        :param mac: (optional) whether get MAC address, default is False.<br>
                if mac == True, the item of result list is MAC address list.<br>
                if mac == 2, the item of result list is a 2-elements tuple of (ip, mac)<br>
                if mac == 3, the item of result list is a 3-elements tuple of (ip, mac, factory)<br>

                :Chinese:
                （可选）是否取得MAC地址，默认值是False.<br>
                如果mac=True,则结果列表每一项是MAC地址。<br>
                如果mac=2, 则结果列表每一项是一个tuple, 形如：(IP地址, MAC地址)。<br>
                如果mac=3, 则结果列表每一项是一个tuple, 形如：(IP地址, MAC地址, 厂商)。<br>

        :param exists:  (optional) Whether get pingable IPs. if exists is False, return list is not pingable IPs.
                :Chinese:（可选）是否取得Ping通的地址，默认值是True.<br>
                如果exists=False,则结果列表是PING不通的IP地址。<br>
        :param threads:   (optional) how many threads that uses. default is 0 for auto detect.
                :Chinese:（可选）并发线程数量，默认值是0(自动选择线程数量）
        :return:  return a list. the item of list depends on the mac parameter.
        """
        # 创建一个扫描任务
        task = _ScanTask(ip_list, mac, exists)
        total = len(ip_list)
        if total == 0:
            return []

        # detect thread count
        if threads <= 0:
            threads = task.get_threads(total)

        # create each thread
        start = 0    # the start index of the ip_list for the thread
        thread_pool = []
        rprint('\rScanning...', end='')
        for i in range(0, threads):
            end = start + int(round(total / threads))  # the end index of the ip_list for the thread
            if i == threads - 1:
                end = total
            # create thread
            t = threading.Thread(target=_ip_scan_thread, args=(task, start, end))
            thread_pool.append(t)  # add to pool
            t.start()
            start = end  # next start point

        # wait for all threads end.
        while True:
            alive = False
            for i in range(len(thread_pool)):
                alive = alive or thread_pool[i].is_alive()
            if not alive:
                break

        task.clear_progress()  # clear progress bar

        return task.result

    @staticmethod
    def is_port_open(ip, port, udp=False, timeout=2000):
        """
        Whether specified port is opened on specified IP address.

        :Chinese: 判断 指定IP地址的指定端口是否开放.

        :param ip:    the IP address or host name
        :param port:  port number, int
        :param udp:   (optional) whether UDP protocol is used. default is False
        :param timeout:   (optional) timeout in milliseconds. default is 2000
        :return: return True if the port is opened. return False if not opened.
        """
        family = socket.SOCK_DGRAM if udp else socket.SOCK_STREAM
        sock = socket.socket(socket.AF_INET, family)
        sock.settimeout(timeout)

        if not isinstance(port, int):
            port = int(port)

        # noinspection PyBroadException
        try:
            state = sock.connect_ex((ip, port))
            sock.close()
            return state == 0
        except Exception:
            return False

    @staticmethod
    def port_scan(ip, port_list=None, exists=True, threads=0, name=False):
        """
        Scan ports of specified IP address.

        :Chinese: 对于指定IP地址，扫描端口列表， 取得 可连接的 端口列表

        :param ip:         the IP address.
        :param port_list:  (optional) a list of port. If port_list is None, scan common used ports.
                :Chinese: (可选)端口列表, 如果port_list为None，则扫描常见端口
        :param exists:   (optional) Whether return opened ports. If exists is False, return not-opened ports.
                :Chinese:（可选）是否取得可连接的端口，默认值是True.<br>
                如果exists=False,则结果列表是不可连接的端口。<br>
        :param threads:  (optional) how many threads that uses. default 0 is auto-detect.
                    :Chinese:（可选）并发线程数量，默认值是0(自动选择线程数量）
        :param name:    (optional) whether return the protocol name.
                        If name is True, each item of result list is a tuple of (port, protocol).
                :Chinese:（可选）是否返回协议名，默认值是False.
                如果name=True,则结果列表每一项是一个tuple, 形如：(端口号, 协议名称)。<br>
        :return:  return a list.
        """
        if not Net.is_ip(ip):
            raise TypeError("%s is not a ip" % repr(ip))

        if port_list is None:
            port_list = Port.common_ports()

        if isinstance(port_list, int):
            port_list = [port_list]

        # create a scan task
        task = _ScanTask(port_list, ip, exists)
        total = len(port_list)
        if total == 0:
            return []

        # detect thread count
        if threads <= 0:
            threads = task.get_threads(total)

        # create each thread
        start = 0  # the start index of the port_list for the thread
        thread_pool = []  # 线程池
        rprint('Scanning...\r', end='')
        for i in range(0, threads):
            end = start + int(round(total / threads))  # the end index of the port_list for the thread
            if i == threads - 1:
                end = total
            # 创建线程
            t = threading.Thread(target=_port_scan_thread, args=(task, start, end))
            thread_pool.append(t)
            t.start()
            start = end  # next start point

        # wait all threads end
        while True:
            alive = False
            for i in range(len(thread_pool)):
                alive = alive or thread_pool[i].is_alive()
            if not alive:
                break

        task.clear_progress()  # clear progress bar

        if name:
            # if name is true, get protocol by port
            ret = []
            for port in task.result:
                protocol = Port.name_of(port)
                ret.append((port, protocol))
            return ret
        else:
            return task.result


# noinspection PyPep8Naming
# noinspection SpellCheckingInspection
# Common used port
class Port:
    """ common used port """

    Echo = 7  # Echo service
    FTP_Data = 20  # File Transfer Protocol data transfer
    FTP = 21  # File Transfer Protocol (FTP) control connection
    SSH = 22  # Secure Shell, secure logins, file transfers (scp, sftp), and port forwarding
    Telnet = 23  # Telnet protocol—unencrypted text communications
    SMTP = 25  # Simple Mail Transfer Protocol, used for email routing between mail servers
    WINS = 42  # WINS Replication
    WHOIS = 43  # WHOIS
    DNS = 53  # Domain Name System name resolver
    TFTP = 69  # Trivial File Transfer Protocol
    HTTP = 80  # HTTP v1, v2 uses TCP.  HTTP v3 uses QUIC/UDP
    Kerberos = 88  # Network authentication system
    MSExchange = 102  # MS Exchange
    POP3 = 110  # Post Office Protocol, version 3 (POP3)
    NNTP = 119  # NNTP (Usenet)
    MSRPC = 135  # Microsoft End Point Mapper, manage services including DHCP server, DNS server, and WINS, DCOM
    NetBIOS = 137  # NetBIOS Name Service, used for name registration and resolution
    NetBIOS_SMB = 139  # NetBIOS Session Service
    IMAP = 143  # Internet Message Access Protocol (IMAP)
    SNMP = 161  # SNMP
    BGP = 179  # BGP
    HP_Openview = 381  # HP data alarm manager
    HP_Openview2 = 383  # HP data alarm manager
    LDAP = 389  # LDAP
    HTTPS = 443  # HTTPS v1, v2 uses TCP. HTTP v33 uses QUIC/UDP.
    SMB = 445  # Microsoft File sharing:Server Message Blocks name
    Kerberos2 = 464  # Kerberos Change/Set password
    SMTPS = 465  # Authenticated SMTP over TLS/SSL (SMTPS)
    RTSP = 554  # RTSP
    NNTPS = 563  # NNTP over SSL
    IMAPS2 = 585  # Secure IMAP
    SMTPS2 = 587  # Email message submission
    MSDOM = 593  # Microsoft DCOM, HTTP RPC Ep Map
    LDAPS = 636  # Lightweight Directory Access Protocol over TLS/SSL
    MPLS = 646  # LDP (MPLS)
    MSExchange2 = 691  # MS Exchange Routing
    Vmware = 902  # VMware ESXi
    FTPS = 989  # FTPS Protocol (data), FTP over TLS/SSL
    FTPS2 = 990  # FTPS Protocol (control), FTP over TLS/SSL
    IMAPS = 993  # Internet Message Access Protocol over TLS/SSL (IMAPS)
    POP3S = 995  # Post Office Protocol 3 over TLS/SSL
    RPC = 1025  # Microsoft Remote Procedure Call
    OpenVPN = 1194  # OpenVPN
    WASTE = 1337  # WASTE Encrypted File Sharing Program
    SqlServer = 1433  # Microsoft SqlServer DB
    VQP = 1589  # Cisco VLAN Query Protocol (VQP)
    PPTP = 1723  # MS PPTP
    Steam = 1725  # Valve Steam Client uses port 1725
    RADIUS = 1812  # RADIUS authentication
    RADIUS2 = 1813  # RADIUS authentication
    NFS = 2049  # Network File System
    cPanel = 2082  # cPanel default
    radsec = 2083  # Secure RADIUS Service (radsec), cPanel default SSL
    Oracle = 2483  # Oracle database listening for insecure client connections to the listener, replaces port 1521
    Oracle2 = 2484  # Oracle database listening for SSL client connections to the listener
    SymantecAV = 2967  # Symantec System Center agent (SSC-AGENT)
    XBOX = 3074  # Xbox LIVE and Games for Windows – Live
    MySQL = 3306  # MySQL database system
    RDC = 3389  # default Remote Desktop
    Warcraft = 3724  # Some Blizzard games, Unofficial Club Penguin Disney online game for kids
    GoogleDesktop = 4664  # Google Desktop Search
    eMule = 4672  # eMule
    SynologyNAS = 5000  # Synology NAS
    UPnP = 5000  # UPnP
    SIP = 5060  # Telephone Session initiation protocol
    PostgreSQL = 5432  # PostgreSQL database system
    VNC = 5900  # virtual Network Computing (VNC) Remote Frame Buffer RFB protocol
    IRC = 6665  # Internet Relay Chat
    IRC2 = 6669  # Internet Relay Chat
    BitTorrent = 6881  # BitTorrent is part of the full range of ports used most often
    BitTorrent2 = 6999  # BitTorrent is part of the full range of ports used most often
    Quicktime = 6970  # QuickTime Streaming Server
    HTTP2 = 8080  # alternate HTTP port (also a Tomcat port)
    KasperskyAV = 8086  # Kaspersky AV Control Center
    KasperskyAV2 = 8087  # Kaspersky AV Control Center
    VMwareServer = 8222  # VMware Server Management User Interface (insecure Web interface).
    HPJetDirect = 9100  # HP JetDirect
    SCP = 9999  # SCP
    BackupExec = 10000  # Webmin, Web-based Unix/Linux system administration tool (default port)
    NetBus = 12345  # NetBus remote administration tool (often Trojan horse).
    Sub7 = 27374  # Sub7 default
    BackOrifice = 18006  # Back Orifice 2000 remote administration tools

    @staticmethod
    def common_ports():
        """ return a list of common used ports """
        result = []
        for name in Port.__dict__:
            if isinstance(Port.__dict__[name], int):
                result.append(Port.__dict__[name])
        return result

    @staticmethod
    def name_of(port_number):
        """ get the name of protocol of specified port number """
        for name in Port.__dict__:
            if Port.__dict__[name] == port_number:
                return name
        return ''



from net import *


w = Spider()
print(w.search("嘌呤").find("是"))
print(Spider().search("嘌呤").find("是"))
print("")

b = ZhiDao("李白 出生地")
print(b.count(), "条答案")
print(b.answer())
# print(ZhiDao("李白 出生地").answer())
print("")

b = BaiKe("爱在深秋", "谭咏麟")
print(b.catalog)
if '歌词' in b:
    print(b['歌词'])
# print(BaiKe("爱在深秋", "谭咏麟")['歌词'])
print("")

b = BaiKe("静夜思")
print(b['全文', '原文'])
# print(BaiKe("静夜思")['全文', '原文'])
print("")

b = WebImage('黑洞', 'png', count=5)
print(b.download_all())
# for index, img in enumerate(b.images):
#     print(img.filename, img.file_ext, img.url)
#     if not img.download(index):
#         print("error downloading image of %s" % index)


# username = ''
# password = ''
# mail = Mail(username, password)



# Send e-mail example

from net import Mail

# Set parameters
username = 'xxxxxxx@host.com'  # username to login SMTP server
password = 'xxxxxxxxxx'      # password to login SMTP server
receiver = 'xxxxxxxx@host.com'  # receiver e-mail address

mail = Mail(username, password)  # Create mail object

# Send mail with attachment file '1.jpg'
mail.send([receiver], 'My Subject', "This is body", ['1.jpg'])




from net import *
import unittest


class TestStrUtil(unittest.TestCase):

    def test_match(self):
        self.assertEqual(StrUtil.match(None, None), True)
        self.assertEqual(StrUtil.match(None, '='), False)
        self.assertEqual(StrUtil.match('a=b', None), True)
        self.assertEqual(StrUtil.match('a=b', '='), True)
        self.assertEqual(StrUtil.match('a<b', '='), False)
        self.assertEqual(StrUtil.match('a<b', ('=', '<')), True)
        self.assertEqual(StrUtil.match('<b', ('=', '<')), True)
        self.assertEqual(StrUtil.match('ab', ('=', '<')), False)
        self.assertEqual(StrUtil.match('a<=b', ['<', '=']), True)
        self.assertEqual(StrUtil.match('a=b', ['=', '<']), False)

    def test_split(self):
        self.assertEqual(StrUtil.split(None, None), [])
        self.assertEqual(StrUtil.split("a=b", None), ["a=b"])
        self.assertEqual(StrUtil.split("a=b", "="), ["a", "b"])
        self.assertEqual(StrUtil.split("a b"), ["a", "b"])
        self.assertEqual(StrUtil.split("a  \t b"), ["a", "b"])
        self.assertEqual(StrUtil.split("  a   \tb  c  "), ["a", "b", "c"])
        self.assertEqual(StrUtil.split("  a, b, c", [',', ' ']), ["a", "b", "c"])

    def test_split2(self):
        self.assertEqual(StrUtil.split2(None, None), ('', ''))
        self.assertEqual(StrUtil.split2('a=b', None), ('a=b', ''))
        self.assertEqual(StrUtil.split2('a=b', '='), ('a', 'b'))
        self.assertEqual(StrUtil.split2('a = b', '='), ('a', 'b'))
        self.assertEqual(StrUtil.split2('a < b', ['=', '<']), ('a', 'b'))
        self.assertEqual(StrUtil.split2('ab', '='), ('ab', ''))
        self.assertEqual(StrUtil.split2('ab', '=', first=False), ('', 'ab'))

    def test_split_number_unit(self):
        self.assertEqual(StrUtil.split_number_unit(None), ('', ''))
        self.assertEqual(StrUtil.split_number_unit('33'), (33, ''))
        self.assertEqual(StrUtil.split_number_unit('-8'), (-8, ''))
        self.assertEqual(StrUtil.split_number_unit('-.33'), (-0.33, ''))
        self.assertEqual(StrUtil.split_number_unit('.33'), (0.33, ''))
        self.assertEqual(StrUtil.split_number_unit('23.5'), (23.5, ''))
        self.assertEqual(StrUtil.split_number_unit('33cm'), (33, 'cm'))
        self.assertEqual(StrUtil.split_number_unit('-8cm'), (-8, 'cm'))
        self.assertEqual(StrUtil.split_number_unit('-.33 cm '), (-0.33, 'cm'))
        self.assertEqual(StrUtil.split_number_unit('.33cm'), (0.33, 'cm'))
        self.assertEqual(StrUtil.split_number_unit('23.5cm'), (23.5, 'cm'))

    def test_grep(self):
        line1 = 'Hello, world 888'
        line2 = ' data=5cm, time=33ms'
        line3 = ' data=6cm, time=65ms'
        line4 = ' data=8cm, time=88ms'
        text = line1 + '\n' + line2 + '\n' + line3 + '\n' + line4
        self.assertEqual(StrUtil.grep(None, None), [])
        self.assertEqual(StrUtil.grep(None, '88ms'), [])
        self.assertEqual(StrUtil.grep(text, '888'), [line1])
        self.assertEqual(StrUtil.grep(text, 'data'), [line2, line3, line4])
        self.assertEqual(StrUtil.grep(text, '88ms'), [line4])
        self.assertEqual(StrUtil.grep(text, 'not-exists'), [])
        self.assertEqual(StrUtil.grep(StrUtil.grep(text, 'data'), '33'), [line2])

    def test_is_ip_number(self):
        self.assertEqual(StrUtil.is_ip_number(None), False)
        self.assertEqual(StrUtil.is_ip_number(''), False)
        self.assertEqual(StrUtil.is_ip_number('0'), True)
        self.assertEqual(StrUtil.is_ip_number('255'), True)
        self.assertEqual(StrUtil.is_ip_number('-3'), False)
        self.assertEqual(StrUtil.is_ip_number('888'), False)
        self.assertEqual(StrUtil.is_ip_number(0), True)
        self.assertEqual(StrUtil.is_ip_number(255), True)
        self.assertEqual(StrUtil.is_ip_number(-3), False)
        self.assertEqual(StrUtil.is_ip_number(3.3), False)
        self.assertEqual(StrUtil.is_ip_number(888), False)


class TestNetwork(unittest.TestCase):

    def test_is_ip(self):
        self.assertEqual(Net.is_ip(None), False)
        self.assertEqual(Net.is_ip(''), False)
        self.assertEqual(Net.is_ip('127.0.0.1'), True)
        self.assertEqual(Net.is_ip('abc'), False)
        self.assertEqual(Net.is_ip('127.0.0'), False)
        self.assertEqual(Net.is_ip('127.0.0.258'), False)
        self.assertEqual(Net.is_ip('-27.0.0.1'), False)

    def test_ip_get_set(self):
        self.assertEqual(Net.ip_get('127.0.0.1', 0), 127)
        self.assertEqual(Net.ip_get('127.0.0.1', 3), 1)
        self.assertRaises(TypeError, lambda: Net.ip_get(None, 3))
        self.assertRaises(ValueError, lambda: Net.ip_get('127.0', 3))
        self.assertRaises(ValueError, lambda: Net.ip_get('127.0.0.1', 4))

        self.assertEqual(Net.ip_get(Net.ip_set('127.0.0.1', 0, 188), 0), 188)
        self.assertEqual(Net.ip_get(Net.ip_set('127.0.0.1', 3, 254), 3), 254)
        self.assertRaises(ValueError, lambda: Net.ip_set('127.0', 3, 55))
        self.assertRaises(ValueError, lambda: Net.ip_set('127.0.0.1', 4, 55))

    def test_port_range(self):
        self.assertEqual(len(Net.port_range(10, 10)), 1)
        self.assertEqual(len(Net.port_range(10, 12)), 3)
        self.assertEqual(len(Net.port_range(12, 10)), 3)
        self.assertEqual(13 in Net.port_range(12, 10), False)
        self.assertEqual(10 in Net.port_range(12, 10), True)
        self.assertRaises(ValueError, lambda: Net.port_range(None, 3))
        self.assertRaises(ValueError, lambda: Net.port_range(-3, 3))
        self.assertRaises(ValueError, lambda: Net.port_range(0, 65536))

        self.assertEqual(len(Net.port_range("10-12")), 3)
        self.assertEqual(len(Net.port_range("10,12")), 2)

    def test_ip_range(self):
        self.assertEqual(len(Net.ip_range("192.168.0.1", 1, 10)), 10)
        self.assertEqual("192.168.0.1" in Net.ip_range("192.168.0.1", 1, 10), True)
        self.assertEqual("192.168.0.3" in Net.ip_range("192.168.0.1", 1, 10), True)
        self.assertEqual("192.168.0.10" in Net.ip_range("192.168.0.1", 1, 10), True)
        self.assertEqual("192.168.0.0" in Net.ip_range("192.168.0.1", 1, 10), False)
        self.assertEqual("192.168.0.11" in Net.ip_range("192.168.0.1", 1, 10), False)

        self.assertRaises(TypeError, lambda: Net.ip_range("192.aab.0.1", 1, 10))
        self.assertRaises(ValueError, lambda: Net.ip_range("192.168.0.1", 1, 1880))
        self.assertRaises(ValueError, lambda: Net.ip_range("192.168.0.1", -1, 80))

        self.assertEqual(len(Net.ip_range("192.168.0.*")), 255)
        self.assertEqual(len(Net.ip_range("192.168.0.1-10")), 10)

    def test_local_ip(self):
        self.assertEqual(Net.is_ip(Net.local_ip()), True)

    def test_ping(self):
        ip = Net.local_ip()
        self.assertEqual(Net.ping(ip) >= 0, True)

        mac = Net.get_mac(Net.ip_set(ip, 3, 1))
        # self.assertEqual(len(get_mac(Net.ip_set(ip, 3, 1))) > 0, True)

        ips1 = Net.ip_range(ip, 1, 255)
        result = Net.ip_scan(ips1, mac=3)
        self.assertEqual(len(result) > 0, True)
        ret = [str(item) for item in result]
        print('\n'.join(ret))

        # result = ip_scan(ips1, mac=True)
        # self.assertEqual(len(result) > 0, True)
        # print(result)
        # ips1 = Net.ip_range(ip, 1, 255)
        # result = ip_scan(ips1, mac=2)
        # # print(result)
        # # result = [('a', 'aa'), ('b', 'bb')]
        # ret = [str(item) for item in result]
        # print('\n'.join(ret))


if __name__ == '__main__':
    unittest.main()
